#!/usr/local/bin/perl -w

# A program for emiting makefile dependencies for Java programs.

use strict;

my $java     = shift || usage();
my $classes  = shift || usage();
my @src_dirs = @ARGV;

my %emitted = ();

read_java($java);

sub read_java {
  my ($file) = @_;
  my $pattern = get_pattern(get_dirs($file));
  my $package;

  open(IN, $file) or die "Can't open $file: $!";
  while (<IN>) {
    !$package and /package\s+(.*);$/ and $package = $1;
    if (/^\s*import\s+(.*);$/) {
      my $import = $1;
      $import =~ /^javax?\./ and next;
      if ($import =~ /\.\*$/) {
	print STDERR "*** Ignoring 'import $import' in $file ***\n";
	next; 
      }
      emit($file, $import);
    } elsif (/\b($pattern)\b/) {
      emit($file, "$package.$1");
    }
  }
}

sub emit {
  my ($target, $class) = @_;

  $target =~ s/\.java$/.class/;
  foreach my $src (@src_dirs) {
    $target =~ s/$src/$classes/;
  }

  my $prerequisite = $class;
  $prerequisite =~ s/\./\//g;
  $prerequisite = "$prerequisite.java";

  foreach my $src (@src_dirs) {
    my $full_prerequisite = "$src/$prerequisite";
    my $dep = "$target: $prerequisite\n";
    -e $full_prerequisite and !$emitted{$dep}++ and print $dep;
  }
}

sub get_dirs {
  my ($file) = @_;
  my $dir = $file;
  $dir =~ s/\/[^\/]+$//;
  $dir =~ s/^[^\/]+\///;
  map { "$_/$dir" } @src_dirs;
}

sub get_pattern {
  my (@dirs) = @_;
  my @files;
  foreach my $dir (@dirs) {
    -e $dir && -d _ or next;
    opendir(DIR, $dir) or die "Can't open dir $dir: $!";
    push @files, grep { s/\.java$//; } readdir DIR;
    closedir DIR;
  }
  join '|', @files;
}


sub usage {
  die "usage: perl $0 <java file> <src dirs>\n";
}
