#!/usr/local/bin/perl -w

use strict;

sub usage { die "perl $0 <package dir> <top dirs>\n"; }

my $package_dir = shift || usage();
my $classes     = shift || usage();
my $mkincludes  = shift || usage();

my @top_dirs    = split /\s+/, $ENV{'VPATH'};

my $javafiles = 0;
my %done;
my @lines = ();

foreach my $top (@top_dirs) {
  my $dir = "$top/$package_dir";
  if (-e $dir and -d _) {
    opendir(DIR, $dir) or die "Can't opendir $dir: $!";
    my @files = readdir DIR;
    closedir DIR;
    
    process_files($top, $package_dir, @files);
  }
}

my $pkg = $package_dir;
$pkg =~ tr/\//./;

$javafiles and print "packages += $pkg\n";

print sort @lines;

sub process_files {
  my ($top, $pkg, @files) = @_;
  foreach (@files) {
    /^\.{1,2}$/ and next;
    /\.java$/ and process_java_file($top, $pkg, $_);
    -d "$top/$pkg/$_" and process_subdir($top, $pkg, $_);
  }
}

sub process_java_file {
  my ($top, $pkg, $file) = @_;
  $javafiles++;
  my $base = $file;
  $base =~ s/\.java$//;
  
  push @lines, "classfiles += $classes/$pkg/$base.class\n";
  if ($top eq 'tests' and $base =~ /Test$/) {
    push @lines, "resultfiles += test_results/$pkg/$base.r\n";
  }
  push @lines, "-include $mkincludes/$pkg/$base.d\n";
}

sub process_subdir {
  my ($top, $pkg, $file) = @_;
  my $mk = "$mkincludes/$pkg/$file/GNUmakefile.gen";
  $done{$mk}++ or push @lines, "-include $mk\n";
}




