package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.util.Map;
import java.util.HashMap;

/**
 * A class that holds a scoped set of parameters that are used to
 * initialize tests. Each Parameters object has a reference to a Map
 * that contains a set of override values; typically these values will
 * come from the command line and will override the values for
 * parameters declared in XML files. After checking the overrides, a
 * Parameters object will look in it's own Map and failing that ask
 * it's parent Parameters object if it has a value for a given key.
 *
 * @version $Id$
 */
class Parameters {

  private Map overrides;
  private Map thisScope;
  private Parameters parent;

  public Parameters(Map o, Map t, Parameters p) {
    overrides = o;
    thisScope = t;
    parent = p;
  }

  public String get(String key) {
    if (overrides != null && overrides.containsKey(key)) {
      return (String)overrides.get(key); 

    } else if (thisScope.containsKey(key)) {
      return (String)thisScope.get(key);

    } else {
      return parent != null ? parent.get(key) : null;
    }
  }
}





