package com.javamonkey.utils.parsing;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.io.Reader;
import java.io.IOException;
import java.io.CharArrayWriter;

/**
 * Base class for hand-written recursive decent parsers. Presently is
 * extremely simple minded about buffering: in the constructor it slurps
 * all its input into memory.
 *
 * @version $Id$
 */
public abstract class RecursiveDecentParser {

  private char[] buffer;
  private IntStack marks = new IntStack();
  private int idx = 0;
  private int lastSuccess;

  private String stringValue;

  RecursiveDecentParser(Reader input) throws IOException {
    CharArrayWriter chars = new CharArrayWriter();
    int c;
    while ((c = input.read()) != -1) { chars.write(c); }
    buffer = chars.toCharArray();
    start();
  }

  protected void start() {
    marks.push(idx);
  }

  protected boolean success() {
    lastSuccess = idx;
    return true;
  }

  protected boolean failure() {
    idx = marks.pop();
    return false;
  }

  protected boolean lookingAt(char c) {
    return buffer[idx] == c;
  }
  
  protected void consume() {
    idx++;
  }

  protected String lastMatch() {
    int start = marks.peek();
    return new String(buffer, start, (lastSuccess - start));
  }

  private class IntStack {
    private int[] ints = new int[10];
    private int idx = 0;

    void push(int i) { 
      if (idx == ints.length) {
	int[] newints = new int[((int)(idx * 1.25)) + 1];
	System.arraycopy(ints, 0, newints, 0, idx);
	ints = newints;
      }
      ints[idx++] = i;
    }

    int pop() {
      try {
	return ints[--idx];
      } catch (ArrayIndexOutOfBoundsException aioobe) {
	throw new StackUnderflowException();
      }
    }

    int peek() {
      try {
	return ints[idx - 1];
      } catch (ArrayIndexOutOfBoundsException aioobe) {
	throw new StackUnderflowException();
      }
    }
  }
}
