package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.util.*;

/**
 * Unit test for com.javamonkey.Parameters.
 *
 * @version $Id$
 */
public class ParametersTest extends BaseTest {

  private boolean counting = true;
  private int count = 0;
  private Parameters me;
  private Parameters parent;
  private Parameters grandparent;
  private Map results = new HashMap();

  private String name = ParametersTest.class.getName();

  public ParametersTest() {

    Map overrides = new HashMap();
    Map grandparentMap = new HashMap();
    Map parentMap = new HashMap();
    Map thisScope = new HashMap();

    for (int h = 0; h < 2; h++) {
      for (int i = 0; i < 2; i++) {
	for (int j = 0; j < 2; j++) {
	  for (int k = 0; k < 2; k++) {
	    String base = h + "." + i + "." + j + "." + k;
	    String key = "key" + "_" + base;
	    if (h == 1) overrides.put(key, "O-" + base);
	    if (i == 1) grandparentMap.put(key, "GP-" + base);
	    if (j == 1) parentMap.put(key, "P-" + base);
	    if (k == 1) thisScope.put(key, base);
	  }
	}
      }
    }

    grandparent = new Parameters(overrides, grandparentMap, null);
    parent = new Parameters(overrides, parentMap, grandparent);
    me = new Parameters(overrides, thisScope, parent);

    run();
    setNumberOfTestCases(count);
    setName(name);
    counting = false;

  }

  public void run() {

    for (int h = 0; h < 3; h++) {
      for (int i = 0; i < 2; i++) {
	for (int j = 0; j < 2; j++) {
	  for (int k = 0; k < 2; k++) {
	    String base = h + "." + i + "." + j + "." + k;
	    String key = "key" + "_" + base;

	    if (h == 1) {
	      // In overrides -- should get that version
	      expect(key, "O-" + base);
	    } else if (h == 2) {
	      // No values were added with h == 2, should be null
	      expectNull(key);
	    } else if (k == 1) {
	      // In thisScope and not overrides
	      expect(key, base);
	    } else if (j == 1) {
	      // In parent and not thisScope or overrides
	      expect(key, "P-" + base);
	    } else if (i == 1) {
	      // In grandparent and not parent, thisScope, or overrides
	      expect(key, "GP-" + base);
	    } else if (h == 0 && i == 0 && j == 0 && k == 0) {
	      // Not in any.
	      expectNull(key);
	    }
	  }
	}
      }
    }
  }


  private void expect(String key, String value) {
    if (counting) {
      count++;
    } else {
      Object got = me.get(key);
      if (value.equals(got)) {
	reportSuccess(key + "->" + value, "ok.");
      } else {
	reportFailure(key + "->" + value, "got '" + got + "', not '" +
		      value + "'");
      }
    }
  }

  private void expectNull(String key) {
    if (counting) {
      count++;
    } else {
      Object got = me.get(key);
      if (got == null) {
	reportSuccess(key + "->null", "ok.");
      } else {
	reportFailure(key + "->null", "got: '" + got + "', not null.");
      }
    }
  }
}



